<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Context;

use Dedi\SyliusQuotePlugin\Entity\QuoteInterface;
use Dedi\SyliusQuotePlugin\Exception\QuoteNotFoundException;
use Dedi\SyliusQuotePlugin\Factory\QuoteFactoryInterface;
use Sylius\Component\Channel\Context\ChannelNotFoundException;
use Sylius\Component\Core\Context\ShopperContextInterface;
use Sylius\Component\Core\Model\ChannelInterface;
use Sylius\Component\Core\Model\CustomerInterface;
use Sylius\Component\Currency\Context\CurrencyNotFoundException;
use Sylius\Component\Locale\Context\LocaleNotFoundException;

class ShopBasedQuoteContext implements QuoteContextInterface
{
    private QuoteFactoryInterface $quoteFactory;

    private ShopperContextInterface $shopperContext;

    private ?QuoteInterface $quote;

    public function __construct(QuoteFactoryInterface $quoteFactory, ShopperContextInterface $shopperContext)
    {
        $this->quoteFactory = $quoteFactory;
        $this->shopperContext = $shopperContext;
        $this->quote = null;
    }

    public function getQuote(): QuoteInterface
    {
        if (null !== $this->quote) {
            return $this->quote;
        }

        $quote = $this->quoteFactory->createNew();
        if (!$quote instanceof QuoteInterface) {
            throw new \InvalidArgumentException(sprintf(
                'Expected an instance of "%s", got "%s"',
                QuoteInterface::class,
                get_class($quote)
            ));
        }

        try {
            /** @var ChannelInterface $channel */
            $channel = $this->shopperContext->getChannel();

            $quote
                ->setChannel($channel)
                ->setCurrencyCode($channel->getBaseCurrency()->getCode())
                ->setLocaleCode($this->shopperContext->getLocaleCode());
        } catch (ChannelNotFoundException | CurrencyNotFoundException | LocaleNotFoundException $exception) {
            throw new QuoteNotFoundException('Sylius was not able to prepare the quote.', $exception);
        }

        /** @var CustomerInterface|null $customer */
        $customer = $this->shopperContext->getCustomer();
        if (null !== $customer) {
            $quote->setCustomer($customer);
        }

        $this->quote = $quote;

        return $quote;
    }

    public function reset(): void
    {
        $this->quote = null;
    }
}
