<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Factory;

use Dedi\SyliusQuotePlugin\Entity\QuoteItem;
use Dedi\SyliusQuotePlugin\Entity\QuoteItemInterface;
use Sylius\Component\Product\Model\ProductInterface;
use Sylius\Component\Product\Resolver\ProductVariantResolverInterface;

class QuoteItemFactory implements QuoteItemFactoryInterface
{
    private ProductVariantResolverInterface $productVariantResolver;

    public function __construct(ProductVariantResolverInterface $productVariantResolver)
    {
        $this->productVariantResolver = $productVariantResolver;
    }

    public function createNew(): QuoteItemInterface
    {
        return new QuoteItem();
    }

    public function createForProduct(ProductInterface $product): QuoteItemInterface
    {
        $quoteItem = $this->createNew();
        $variant = $this->productVariantResolver->getVariant($product);

        if (null !== $variant) {
            $quoteItem
                ->setVariant($variant)
                ->setVariantName($variant->getName());
        }

        $quoteItem->setProductName($product->getName());

        return $quoteItem;
    }
}
